
//////////////////////////////////////////////////////////////
//
//          FMotifs
//                      A program for the fast evaluation of
//                      motifs in dense medium-size networks.
//                      Developed by M. Zanin.
//
//                      V2.0
//
//                      Check www.mzanin.com/FMotifs for further
//                      details and usage examples.
//
//                      Download the manual at:
//                      http://www.mzanin.com/FMotifs/FMotifs%20Guide.pdf
//


#ifndef FASTMOTIFS3_H
#define FASTMOTIFS3_H


class FastMotifs3
{
    public:

        //  Creation and destruction of the class
        FastMotifs3();
        virtual ~FastMotifs3();
        void AllocMemory(int numNodes);
        void ClearMemory();

        //  Returns the version of the library
        float GetVersion();

        //  Main calculation function
        void CalculateMotifs();

        //  Functions for parallelization
        void ParCreateIndices(int numCores);
        void ParCalculateMotifs(int idCore);

        //  Network data
        bool *AM;
        int numNodes;

        //  Results storage
        int MotifsArray[512];
        int Motifs[13];

        int *ParIndices;


    protected:
    private:

        //  Private functions, not to be used directly

        //  Convert individual hits to motifs
        void DecryptMotifs();

        //  Fast bit manipulation
        inline int SetBit(int Value, int Offset);
        inline int ResetBit(int Value, int Offset);

        //  Functions for parallelizing calculations
        double ParNumIterations();
        int ParTargetNode(int inNode, double numIterations);
        inline float CubeRoot(float X);

        // Pre-calculate connected nodes
        bool *IsConnected;
        void CalculateConnections();
};

#endif // FASTMOTIFS3_H
